/*
 JackWidget 2.3.5
 http://www.red6ork.org

 Copyright (C) 2005-2011 Dave Richmond
 daver@redwork.org
 
 Released under GNU General Public License
 
 This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

For a copy of the licence, go to 
http://www.gnu.org/copyleft/gpl.html


 portions Copyright (C) 2005, Apple Computer, Inc.
 (preferences button animation)

*/
 
var consoleWindow;
var splitChar = "/";
var queryChar = "?"; 
var tempDir = "/tmp";
var tempFile = "jacktemp";
var popupPref = "lastPopup-";
var widgetPath = location.pathname.substring(0,location.pathname.lastIndexOf("/"));
var debugging = true;
var searchField, searchPopup;

if(window.widget) {
	widget.onremove = onremove;
}

function setup(elem) {
	ptoc("------------------------------------");
	ptoc("window.location =" + window.location);
	
	searchField = document.getElementById("searchField");
	searchPopup = document.getElementById("searchPopup");

	//for (i in searchField) {
	//	ptoc(i + " = " + searchField.i);
	//}

	urlsfile = openXMLfile("urls.xml");
	urls = parseURLs(urlsfile);

	searchesfile = openXMLfile("searches.xml");
	searches = parseSearches(searchesfile);

	// populate popup box choices
	for (i in searches) {
		newOption = new Option(searches[i].caption, searches[i].id);
		searchPopup.add(newOption);
	}
	// window.onkeypress = keyPressed(event);
	
	// if saved search, set the popup box
	if(window.widget) { 
		var lastPopup = widget.preferenceForKey(popupPref + widget.identifier);
		if (lastPopup && lastPopup.length >0) {
			// set the currently selected search to lastSearch
			for (var i=0; i< searchPopup.length; i++) {
				if (lastPopup == searchPopup.options[i].value) {
					searchPopup.options[i].selected=true;
					// update popup attributes
					updateFieldAttrs();
					searchPopup.focus(true);
					break;
				}
			}
		}
	}
	return 0;
}

function parseURLs(file) {
	var urls = new Array();
	var x = file.responseXML.getElementsByTagName("urls");

	//ptoc(enumobjs(document.getElementById("searchField")));
	
	for (i=0; i<x[0].childNodes.length; i++) {
		var nNode = x[0].childNodes[i];
		if (nNode.nodeType != 1) continue;
		var id = nNode.getAttribute("id");
		var type = nNode.getAttribute("type");
		//ptoc("url id=" + id + ", type=" + type);
		newurl = new urlDef();
		fields = new Array();
		values = new Array();
		defaults = new Array();
		fieldcounter = 0
		for (j=0; j<nNode.childNodes.length; j++) {
			var nodeName = nNode.childNodes[j].nodeName;
			if (nodeName != "#text" && nNode.childNodes[j].firstChild != null) {
				var nodeValue =	nNode.childNodes[j].firstChild.nodeValue;
				if (nodeName == "field") fields[fieldcounter++] = nodeValue;
				if (nodeName == "value") values[fieldcounter-1] = nodeValue;
				if (nodeName == "default") defaults[fieldcounter-1] = nodeValue;
				else newurl[nodeName] = nodeValue;
			}
		}
		newurl["numfields"] = fieldcounter;
		newurl["fields"] = fields;
		newurl["values"] = values;
		newurl["defaults"] = defaults;
		newurl["id"] = id;
		newurl["type"] = type;
		urls[newurl["id"]] = newurl;
	}
	//for (i in urls) ptoc("url " + i + " = " + urls[i].numfields);
	return urls;
}


function parseSearches(file) {
	var searches = new Array();
	var x = file.responseXML.getElementsByTagName("searches");
	
	//ptoc(enumobjs(document.getElementById("searchField")));
	
	for (i=0; i<x[0].childNodes.length; i++) {
		var nNode = x[0].childNodes[i];
		if (nNode.nodeType != 1) continue;
		var id = nNode.getAttribute("id");
		
		newsearch = new searchDef();
		urllist = new Array();
		urlcounter = 0
		for (j=0; j<nNode.childNodes.length; j++) {
			var nodeName = nNode.childNodes[j].nodeName;
			if (nodeName != "#text" && nNode.childNodes[j].firstChild != null) {
				var nodeValue =	nNode.childNodes[j].firstChild.nodeValue;
				if (nodeName == "url") urllist[urlcounter++] = nodeValue;
				else newsearch[nodeName] = nodeValue;
			}
		}
		newsearch["id"] = id;
		newsearch["numurls"] = urlcounter;
		newsearch["urllist"] = urllist;
		searches[newsearch["id"]] = newsearch;
	}
	return searches;
}


function popupChanged() {
	// update search field attributes
	updateFieldAttrs();

	// store current search in prefs
	if(window.widget) {
		widget.setPreferenceForKey(searchPopup.value, popupPref + widget.identifier);
	}

	// if text in field give focus back
	if (searchField.value != "") { 
		searchField.focus(true); 
		searchField.select(); 
	}
}

function updateFieldAttrs() {
	// set placeholder to new item
	if (searchPopup.value == "query") {
		searchField.setAttribute("placeholder", "query keywords");
		searchField.setAttribute("title", "query keywords");
	} else {
		searchField.setAttribute("placeholder", searches[searchPopup.value].placeholder);
		searchField.setAttribute("title", searches[searchPopup.value].placeholder);
	}
}

// keyPressed is not used
function keyPressed(event) {
	switch(event.keyCode) {
		case 27:  
			searchField.blur(); 
			break;
		case 9: 
			searchPopup.focus(); 
			break;
		default:
			//searchField.focus();
			break;
	}
}


function search() {
	ptoc("begin - search()");
	var theurl = "";
	var searchid = searchPopup.value;
	var searchString = trim(searchField.value);
	if (searchString.length == 0) return;
	// Detect cli query mode
	// cli invoked by choosing "Query" option in popup (default, it's the first option)
	//     or by prefacing the search string with the defined queryChar (default "?")
	// cli queries look like   "ama harry potter" or "?ama harry potter"
	//  
	if (searchid=="query" || searchString.substring(0,1) == queryChar) { 
		// get chars until first space
		var firstspaceindex = searchString.indexOf(" ");
		var startind;
		// ignore leading character if it's the queryChar (default '?')
		if (searchString.substring(0,1) == queryChar) startind = 1;
		else startind = 0;
		// get the query id, usually a 3-4 char abbr associated with a given search
		prefix = searchString.substring(startind,firstspaceindex);
		// find associated search and set id to associated search
		for (i in searches) {
			if (searches[i].abbreviation == prefix) {
				searchid = searches[i].id; break;
			}
		}
		// trim search string to remove leading query indicator string
		searchString = searchString.substring(firstspaceindex+1,searchString.length);
	}
	// get the urls assoc'ed with this search 
	urllist = searches[searchid].urllist;
	ptoc("searches[searchid]=" + searches[searchid]);
	for (i in urllist) {
		ptoc("i=" + i);
		ptoc("urllist=" + urllist);
		var theurl = "";
		var currurl = urls[urllist[i]];
		ptoc(currurl);
		var replacewith = currurl.replacespacewith;
		if (currurl.numfields == 0) { // non parsed
			theurl = currurl.urlstring + replaceSpaces(searchString, replacewith);
		} else { // search has multiple fields, must parse string into fields
			var splitString = searchString.split(splitChar);
			var numstring = splitString.length;
			var numfields = currurl.fields.length;
			
			// construct array of the final values for each form input
			var thevalues = new Array(numfields);
			var j=0;
			for (var k=0; k<numfields; k++) {
				var theval = currurl.values[k];				
				if (theval == null) theval = splitString[j++];
				if (theval == null) theval = currurl.defaults[k];
				if (theval == null) theval = "";
				// if we're at the last field, stuff all the rest of the input into it
				if (k==numfields-1) {
					for (el=j; el<numstring; el++) {
						theval += " " + splitString[el];
					}
				}
				thevalues[k] = replaceSpaces(trim(theval), replacewith);	
			}
			
			if (currurl.type == "post") { // method="POST"
				var thefile = tempDir + "/" + tempFile + i + ".html";
				widget.system("/bin/rm " + thefile, null);
				theurl = "file://" + thefile;			
				var thehtml = "<html><head></head><body>";
				thehtml += "<form action=\'" + currurl.urlstring + "\' method=\'POST\'>";
				for (var k = 0; k < numfields; k++) {
					thehtml += "<input type=\'hidden\' name=\'" + currurl.fields[k] + 
										 "\' value =\'" + thevalues[k] + "\'>";
				}
				thehtml += "</form><script>document.forms[0].submit();</script></body></html>"
				// now save the file
				widget.system("/bin/echo \"" + thehtml + "\" > " + thefile, null);
			} else { // it's a regular method="get" (or, sometimes method='post' doesn't need special treatment
				theurl = currurl.urlstring;
				for (var k = 0; k < numfields; k++) {
					if (theurl.substring(-1) != "?") theurl += "&";
					theurl += currurl.fields[k] + "=" + thevalues[k];
				}
			}
		}
		// finally, open the url (whew!)
		ptoc("theurl = " + theurl);
		if (window.widget) {			 
			widget.openURL(theurl);
		} else {
			// this is really a debugging option if opened in a browser
			a = window.open("", "", "")
			a.location = theurl;
		}
	}
}


function replaceSpaces(searchString, replaceString) {
	var b = "";
	//if (replaceString == "") replaceString = "+";
	if (replaceString != "") {
		for (i=0; i< searchString.length; i++) {
			if (searchString[i] == " ") b = b + replaceString;
			else b = b + searchString[i];
		}
		return escape(b);
	} else {
		return escape(searchString);
	}
}

function trim(astring) {
	//use regexp to 'unmatch' the beginning and ending whitespace from a string
	if (astring != "" && astring != null) {
		var trimmed = astring.match(/(\S.*\S)|\S/g);
		return (trimmed != "" && trimmed != null? trimmed.toString(): trimmed);
	}
	else return astring;
}

//object definitions
function searchDef() {
	this.id = "";  
	this.abbreviation = "";
	this.caption = "";
	this.placeholder = ""; 
	this.urllist = new Array(); 
}

function urlDef() {
	this.id = "";
	this.type = "";
	this.urlstring = "";	
	this.fields = new Array();
	this.values = new Array();
	this.numfields = 0;
	this.replacespacewith = ""; // string to replaces spaces with
}


// XML routines
function openXMLfile(name) {
	req = new XMLHttpRequest();
	req.onreadystatechange = processReqChange;
	req.overrideMimeType("text/xml");
	req.open("GET", name, false);
	req.setRequestHeader("Cache-Control", "no-cache");
	req.send(null);
	return req
}


// handle onreadystatechange event of req object
function processReqChange() {
    // only if req shows "loaded"
    if (req.readyState == 4) {
        // only if "OK"
        if (req.status != 200) {
            alert("There was a problem retrieving the XML data:\n" +
                req.status + " : " + req.statusText);
         }
    }
}

function getHelp() {
	var loc = window.location.toString();
	var firstslash = loc.indexOf("/");
	var path = loc.slice(0, firstslash) + "//" + loc.slice(firstslash, loc.lastIndexOf("/") + 1);
	var theurl = path + "JWDoc.html";
	if (window.widget) {
		widget.openURL(theurl);
	} else {
		location = theurl;
	}
}

// debugging
function ptoc(aMessage) {
	if (!window.widget && debugging) {
		if (consoleWindow == null) {
			consoleWindow = window.open("", "tinyWindow", 'width=500,height=800') 
		}
		consoleWindow.document.write(aMessage + "<br>");
	}
}

function enumobjs(a) {
	s="";
	for (i in a) {
		s += i + " = " + a.i + "<br>";
	}	
	return s;
}

function mouseOver(thing) {
	thing.style.textDecoration='underline';
	thing.style.cursor = 'pointer';
}
	
function mouseOut(thing) {
	thing.style.textDecoration='none';
	thing.style.cursor = 'auto';
}

function onremove() {
	if(window.widget) {
		widget.setPreferenceForKey(null, popupPref + widget.identifier);
	}
}


//
// Dave's note:
// The remainder of the code was taken from Apple's widget programming examples
//

// front/back side flipping	
function showPrefs()
{
	// first, you need to get the front
	var front = document.getElementById("front");		
	// and back layers
	var back = document.getElementById("back");			
	// this freezes the currently visible layer
	if(window.widget)									
		widget.prepareForTransition("ToBack");
	// the preferences layer is made
	front.style.display="none";							
	// visible and the picture is hidden
	back.style.display="block";							
	// the flip transition is run, with the	frozen side on the front and the		
	if(window.widget)									
		setTimeout ('widget.performTransition();', 0);	
	// preferences on the back													
	document.prefButtonHighlight.style.display='none';
}

/* When the "Done" button is clicked in the preferences, 
   this function is called. It swaps the preference layer 
   out with the picture and frame.
*/ 
function hidePrefs()
{
	
	document.getElementById("doneButton").src = "Images/done.png";  
	// restore the regular "Done" button
	// again, you need to obtain the front
	var front = document.getElementById("front");	
	// and back layers
	var back = document.getElementById("back");		
	// then freeze the visible layer (as seen by the user
	if (window.widget)				
		widget.prepareForTransition("ToFront");			
	// hide the preferences layer and shot the
	front.style.display="block";
	// main picture layer 
	back.style.display="none";		
	// and run the flip transition; note that since
	// finishEdit() was used to freeze the UI, this
	if (window.widget) setTimeout ('widget.performTransition();', 0);	 
}


